<?php
/* vim :set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * Tirzen Framework (TZN)
 *
 * This declares the common class from which any object from the TZN shall
 * inherit. It is compatible with PHP versions 4 and 5.
 *
 * THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
 * MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 * This package is licensed under the LGPL License
 * Copyright (C) 2005 Stan Ozier
 *
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version.
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA 
 *
 * @package    Tirzen Framework
 * @author     Stan Ozier <stan@tirzen.net>
 * @copyright  2005 - Stan Ozier
 * @license    http://www.gnu.org/licenses/lgpl.txt (LGPL)
 * @link       http://www.tirzen.net/tzn/
 * @version    1.1
 */
 
/**
 * TZN: Tirzen Framework (TZN) common/generic class
 *
 * @package    TZN
 * @author     Stan Ozier <stan@tirzen.net>
 * @version    1.1
 */



/* ------------------------------------------------------------------------ *\
define("TZN_DEBUG",0);
	// 0 no debug, 1 on error, 2 show most, 3 show all
define("TZN_SPECIALCHARS",1);
	// 0 = none, 1 = custom > & ", 2 = htmlspecialchars, 3 = htmlentities
define("TZN_HTMLMODE","html");
	// no = no HTML, bbs = BBS [b]style[/b], html = HTML accepted
define("TZN_BOOL_TRUE","Y");
define("TZN_BOOL_FALSE","N");
	// image or HTML for booleans
define("TZN_TZDEFAULT","user");
define("TZN_DATEFIELD","SQL");
\* ------------------------------------------------------------------------ */
define("TZN_DATE_SQL","%Y-%m-%d");
define("TZN_DATE_FRM","%d %b %y");
// define("TZN_DATE_FRM","%d/%m/%y"); // EUR
// define("TZN_DATE_FRM","%m/%d/%y"); // USA
define("TZN_DATE_SHT","%d %b %y"); // -TODO- %e bug
define("TZN_DATE_SHX","%a %e %b %y");
define("TZN_DATE_LNG","%d %B %Y");
define("TZN_DATE_LNX","%A %d %B %Y");
define("TZN_DATETIME_SQL","%Y-%m-%d %H:%M:%S");
define("TZN_DATETIME_EUR","%d/%m/%y %H:%M");
define("TZN_DATETIME_USA","%m/%d/%y %I:%M%p");
define("TZN_DATETIME_SHT","%d %b %y %H:%M");
define("TZN_DATETIME_SHX","%a %d %b %y %H:%M");
define("TZN_DATETIME_LNG","%d %B %Y, %H:%M");
define("TZN_DATETIME_LNX","%A %d %B %Y, %H:%M");

define("TZN_KEY_LENGTH",8);
define("TZN_KEY_STRING","ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890");

define("TZN_NOW",strtotime(gmdate('Y-m-d H:i:s')));
define("TZN_TZSERVER",date('Z'));

if (@constant('TZN_TRANS_ID')) {
	ini_set("session.use_trans_sid",1);
}

class Tzn {

	var $_error;
	var $_properties;
	
	function Tzn() {
        $this->_error = array();
    }
    
    function addProperties($arrProps) {
    	if (is_array($arrProps)) {
			if (is_array($this->_properties)) {
				$this->_properties = array_merge($this->_properties,$arrProps);
			} else {
				$this->_properties = $arrProps;
			}
		}
    }
	
	/* --- GET  -------------------------------------------------- */
	
	/**
	 * get : generic function
	 * check property type and call corresponding method
	 * variable parameters
	 */
	function get($key)
	{
		if (!$key) {
			return 'Error: Tzn::get (empty 1st parameter)';
		}
		if ($dfn = $this->_properties[$key]) {
			$arrType = explode(',',$dfn);
			if ($arrType[0] == 'OBJ') {
				// indirect call to object
				$arrArgs = func_get_args();
				array_shift($arrArgs[0]); // remove first parameter (key)
				$obj = $this->$key;
				if (is_object($obj)) {
					return call_user_func_array(array(&$obj,'get'), $arrArgs);
				}
			} else if (func_num_args() > 1) {;
				$arrArgs = func_get_args();
				return call_user_func_array(array(
					&$this,'get'.$arrType[0]),$arrArgs);
			} else {
				$type = $arrType[0];
				$arrType[0] = $key;
				// echo "calling get$type with "; print_r($arrType);
				return call_user_func_array(array(&$this,'get'.$type),$arrType);
			}
		}
	}
	
	function getUid() {
		return $this->id;
	}
	
	function getRdm($len = TZN_KEY_LENGTH, $strChars = TZN_KEY_STRING)
	{
		$strCode = "";
		$intLenChars = strlen($strChars);
		for ( $i = 0; $i < $len; $i++ )	{
				$n = mt_rand(1, $intLenChars);
				$strCode .= substr($strChars, ($n-1), 1);
		}
		return $strCode;
	}
	
	function getInt($keyval,$default=0)
	{
		$value = Tzn::_value($keyval,$default);
		return number_format(round($value),0);
	}

	function getNum($keyval,$default=0)
	{
		$value = Tzn::_value($keyval,$default);
		$value = number_format(abs(round($value)),0);
		if (!$value) {
			$value = $default;
		}
		return $value;
	}
	
	/**
	 * getDec (key/val,[decimal],[default])
	 */
	function getDec($keyval)
	{
		$arrArgs = func_get_args();
		$i = 1;
		$dec = null;
		if (intval($arrArgs[1]) == $arrArgs[1]) {
			// number of digits after decimal point
			$dec = $arrArgs[1];
			$i++;
		}
		$value = Tzn::_value($keyval,$arrArgs[$i]);
		if ($dec) {
			return number_format($value,$dec);
		} else {
			if (round($value) != $value) {
				$dec = 2;
				if (is_object($this)) {
					$arrType = explode(',',$this->_properties[$key]);
					if (count($arrType) > 1) {
						$dec = $arrType[1];
					}
				}
				return number_format($value,$dec);
			}
			return number_format($value);
		}
	}

	function getDec2($keyval)
	{
		$arrArgs = func_get_args();
		$i = 1;
		$dec = null;
		if (intval($arrArgs[1]) == $arrArgs[1]) {
			// number of digits after decimal point
			$dec = $arrArgs[1];
			$i++;
		}
		$value = Tzn::_value($keyval,$arrArgs[$i]);
		if ($dec) {
			return number_format($value,$dec,'.','');
		} else {
			if (round($value) != $value) {
				$dec = 2;
				if (is_object($this)) {
					$arrType = explode(',',$this->_properties[$key]);
					if (count($arrType) > 1) {
						$dec = $arrType[1];
					}
				}
				return number_format($value,$dec,'.','');
			}
			return number_format($value,null,'.','');
		}
	}
	
	/**
	 * getStr (key/val,[cut],[default])
	 */
	function getStr($keyval)
	{
		$arrArgs = func_get_args();
		return Tzn::_strValue($keyval,$arrArgs[1],$arrArgs[2]);
	}
	
	function getUrl($keyval,$default='') {
		return Tzn::_value($keyval,$default);
	}
	
	function getEml($keyval,$default='') {
		return Tzn::_value($keyval,$default);
	}
	
	function getTxt($keyval)
	{
		$arrArgs = func_get_args();
		return Tzn::_strValue($keyval,$arrArgs[1],$arrArgs[2]);
	}
	
	function getBbs($keyval,$default='')
	{
		return Tzn::_value($keyval,$default);
	}
	
	function getHtm($keyval,$default='')
	{
		return Tzn::_value($keyval,$default);
	}
	
	function getBol($keyval,$default=null)
	{
		if (is_object($this)) {
			$value = Tzn::_value($keyval,$default);
		} else {
			$value = $default;
		}
		return $value?true:false;
	}
	
	function getDte($keyval,$format='SQL',$default='')
	{
		$value = Tzn::_value($keyval,$default);
		if (!$value || ereg('^(0000|9999)',$value)) {
			return $default;
		} else if (($ts = strtotime($value)) > 0) {
			$const = 'TZN_DATE_'.$format;
			if (defined($const)) {
				$format = constant($const);
			} else if (!ereg('%',$format)) {
				$format = TZN_DATE_SQL;
			}
			return strftime($format,$ts);
		} else {
			// error
			return '/!\\';
		}
	}
	
	function getDtm($keyval,$format='SQL',$tz=TZN_TZDEFAULT,$default='')
	{
		$value = Tzn::_value($keyval,$default);
		return Tzn::_dtmValue($value,$format,$tz,$default);
	}
	
	/* -- Private methods ---- */
	
	/**
	 * z : static function to retreive data (object or static)
	 */
	
	function _value($keyval,$default='')
	{
		if (is_object($this)) {
			if (preg_match("/^(.*)\((.*)\)$/i",$keyval,$m)) {
				$keyval = $m[1];
				if ($m[2]) {
					$value = call_user_func_array(array(&$this,$keyval),
						explode(',',$m[2]));
				} else {
					$value = $this->$keyval();
				}
			} else {
				if (is_numeric($keyval)) {
					$value = $keyval;
				} else {
					$value = $this->$keyval;
				}
				if (TZN_DEBUG == 3) {
					echo '['.$keyval.']=['.$value.']';
				}
			}
		} else {
			$value = $keyval;
		}
		//if (is_null($value)) {
		// echo '['.$value.'-'.$default.']';
		if (empty($value)) {
			$value = $default;
		}
		return $value;
	}
	
	function _strValue($keyval)
	{
		$arg = func_get_args();
		if (func_num_args() > 1) {
			if ((intval($arg[1])) === $arg[1]) {
				// cut the string
				$cut = $arg[1];
				$default = $arg[2];
			} else {
				$default = $arg[1];
			}
		} else {
			$default ='';
		}
		$value = Tzn::_value($keyval,$default);
		$value = strip_tags($value);
		if ($cut) {
			$value = str_replace("\r\n"," ",$value);
			if (($cut > 2) && (strlen($value) > $cut)) {
				$value = trim(substr($value,0,($cut-2))).".."; 
			}
		}
		switch (TZN_SPECIALCHARS)
		{
			case 1:
				$spe = array('&','<','>','"');
				$sfe = array('&amp;','&lt;','&gt;','&quot;');
				$value = str_replace($spe,$sfe,$value);
				break;
			case 2:
				$value = htmlspecialchars($value);
				break;
			case 3:
				$value = htmlentities($value);
				break;
		}
		return $value;
	}

	function _getUserTZ($tz=TZN_TZDEFAULT) {
		if ($tz == 'server') {
			$tz = TZN_TZSERVER;
		} else if ($tz == 'user') {
			$tz = intval($_SESSION['tznUserTimeZone']);
		} else if (is_string($tz)) {
			$tz = $this->getInt($tz);
		} else {
			$tz = intval($tz);
		}
		return $tz;
	}

	function _adjustUserDte($timestamp) {
		$servNow = strftime(TZN_DATE_SQL);
		$userNow = strftime(TZN_DATE_SQL,time() - TZN_TZSERVER + $this->_getUserTZ());
		error_log('GMT: '.strftime(TZN_DATE_SQL,TZN_NOW).', server: '.$servNow.', user: '.$userNow);
		if ($servNow < $userNow) {
			$timestamp += 86400;
		} else if ($userNow < $servNow) {
			$timestamp -= 86400;
		}
		error_log(strftime(TZN_DATE_SQL,$timestamp));
		return $timestamp;
	}
	
	function _dteValue($value,$format='SQL',$default='')
	{
		$const = 'TZN_DATE_'.$format;
		if (defined($const)) {
			$format = constant($const);
		} else if (!ereg('%',$format)) {
			$format = TZN_DATE_SQL;
		}
		if (!$value || ereg('^(0000|9999)',$value)) {
            // case 1: no date set
            return '9999-00-00';
		} else if ($value == 'now' || $value == 'NOW') {
			// case 2: now
			$value = time() - TZN_TZSERVER + $this->_getUserTZ();
		} else if ($wday = array_search(strtolower($value),$GLOBALS['arrLanguageDateDay'])) {
            // case 3: week day (monday - sunday)
            $value = strtotime($wday);
		} else if (ereg('('.implode('|',$GLOBALS['arrLanguageDate']).')',strtolower($value))) {
            // case 4: keyword (day, week...)
			if (FRK_LANGUAGE != 'en') {
				$value = str_replace(
					array_values($GLOBALS['arrLanguageDate']),
					array_keys($GLOBALS['arrLanguageDate']),
					$value);
			}
			$value = $this->_adjustUserDte(strtotime($value));
		} else if (@strtotime($value) === -1) {
			// case 5: no value
			if (ereg('^[0-9]{4}\-[0-9]{2}\-[0-9]{2}',$default)) {
				return $default;
			} else {
				$value = strtotime($default);
			}
		} else if (!ereg('^[0-9]{4}\-[0-9]{2}\-[0-9]{2}',$value)) {
			// case 6: another date format eg. 12 apr
			$value = strtotime($value);
		} else {
			// case 7: already SQL formatted date
			$year = intval(substr($value,0,4));
			if ($year == 9999 || $year == 0) {
				if (($dt = @strtotime($default)) === -1) {
					$value = $default;
				} else {
					$value = $dt;
				}
			} else if ($year >= 1970) {
				$value = $this->_adjustUserDte(strtotime($value));
			} else {
				if (($year >= 1902) && ($year <= 1951)) {
					$mapyear = $year + 84;
				} else if (($year >= 1952) && ($year <= 1969)) {
					$mapyear = $year + 28;
				} else {
					return $default;
				}
				$format =& str_replace (array ("%y", "%Y"),
					array (substr ($year, -2), $year), $format);
				$value = strtotime($mapyear.substr($value,4));
			}
		}		
		return strftime($format,$value);
	}
	
	function _dtmValue($value,$format='SQL',$tz=TZN_TZDEFAULT,$default='')
	{
		if ($value == 'now' || $value == 'NOW') {
			$value = TZN_NOW;
		} else if ((ereg('^(0000|9999)',$value)) 
			|| (($dt = @strtotime($value)) === -1)) 
		{
			if (!ereg('^[0-9]{4}\-[0-9]{2}\-[0-9]{2}',$default)) {
				return $default;
			} else {
				$value = $default;
			}
		} else {
			$value = $dt;
		}
		if (!$value) {
			return '0000-00-00';
		}
		$tz = $this->_getUserTZ($tz);
		$const = 'TZN_DATETIME_'.$format;
		if (defined($const)) {
			$format = constant($const);
		} else if (!ereg('%',$format)) {
			$format = TZN_DATETIME_SQL;
		}
		return strftime($format,$value+$tz);
	}

	function dump($nested = '')
	{
       	// setting data coming from an array (GET, POST, SESSION or mySQL)
        foreach($this->_properties as $key => $type) {
        	if ($type == 'OBJ') {
				echo $nested.$key." [$type] {\r\n";
				$this->$key->dump($nested.'  ');
				echo $nested."} \r\n";
        	} else {
				echo $nested.$key." [$type] => "
					.((is_null($this->$key))?'NULL':$this->$key)."\r\n";
			}
        }
    }
	
	/* --- SET  -------------------------------------------------- */

	function initObjectProperties($nested = false) {
		foreach($this->_properties as $key => $type) {
			if ($type == 'OBJ') {
				if ($nested) {
					$this->$key = new $key();
				} else {
					if (!is_object($this->$key)) {
						$this->$key = new $key();
					}
					$this->$key->initObjectProperties(true);
				}
			}
		}
	}
	
	function setAuto(&$data, $nested = '')
	{
       	// setting data coming from an array (GET, POST, SESSION or mySQL)
       	$method = '_setFromHttp';
        if (is_object($data)) {
        	if (TZN_DEBUG == 3 && !$nested) {
        		echo "<pre>**** DATA - BEGIN *****\r\n";
				var_dump($data);
				echo '**** DATA - END ****</pre>';
			}
        	$method = '_setFromObject';
        }
        foreach($this->_properties as $key => $type) {
        	if ($type == 'LVL' && ($method == '_setFromHttp')) {
        		continue;
        	} else if ($key == 'id') {
        		if ($nested) {
					$this->$method($data,'id',$nested.'Id');
				} else if (is_object($data) || @constant('TZN_PHP4_SQLITE')) {
					$this->$method($data,'id',$this->getIdKey());
        		} else {
					$this->$method($data,'id','id');
				}
        	} else if ($type == 'OBJ') {
        		if ($nested) {
        		// set only id of 2nd level nested object
					$obj = new $key();
					$obj->$method($data,'id',$nested.'_'.$key.'Id');
					// $nid = $nested.'_'.$key.'Id';
					// echo "setting $key ID=".$data->$nid." to $key by $method <br/>";
					$this->$key = $obj;
					unset($obj);
				} else {
					if (!is_object($this->$key)) {
						$this->$key = new $key();
					}
					$this->$key->setAuto($data,$key);
				}
        	} else if ((eregi('^IMG',$type)) && ($method == '_setFromHttp')) {
        		// upload
        		$arrType = explode(',',$type);
        		$arrType[0] = $key;
        		call_user_func_array(array(&$this,'uplImg'),$arrType);
        	} else {
        		$nKey = $nested?$nested.'_'.$key:$key;
				$this->$method($data,$key,$nKey);
				/*
				if ($nested && ($method == '_setFromHttp')) {
					echo "$nested: setting $key as ".$this->$key." ($nKey) <br/>";
				}
				*/
			}
        }
    }
    
    function _setFromHttp(&$data,$key,$dkey) {
    	if (isset($data[$dkey])) {
    		$value = $data[$dkey];
			if (get_magic_quotes_gpc() && $value) {
				// remove fucking magic quotes
				$value=stripslashes($value);
			}
			$this->set($key,$value);
		}
    }
    
    function _setFromObject(&$data,$key,$dkey) {
    	$this->$key = $data->$dkey;
    }

	function setHttp($key,$value) {
		if (!is_null($value)) {
			if (get_magic_quotes_gpc() && $value) {
				// remove fucking magic quotes
				$value=stripslashes($value);
			}
			$this->set($key, $value);
			/*
			$arrType = explode(',',$this->_properties[$key]);
			if ($key == 'id') {
				$this->setUid($value);
			} else if ((!is_null($value)) || ($arrType[0] == 'IMG')) {
				$arrArgs = func_get_args();
				echo "calling... set".$arrType[0]." with <pre>";
				print_r($arrArgs);
				echo "</pre>";
				exit;
			}
			*/
		}
	}
    
    function set($key,$value)
	{
		$arrType = explode(',',$this->_properties[$key]);
		if ($key == 'id') {
			$this->setUid($value);
		} else if ((!is_null($value)) || ($arrType[0] == 'IMG')) {
			$arrArgs = func_get_args();
			if ($arrType[0]) {
				return call_user_func_array(array(
					&$this,'set'.$arrType[0]),$arrArgs);
			} else {
				echo "OOPS: error calling set() (no type defined for $key)";
				exit;
			}
		}
	}
    
    function setUid($value) {
    	if (ereg('^[0-9a-zA-Z]*$',$value)) {
	    	$this->id = $value;
	    }
    }
	
	function setInt($key, $value)
	{
		$value = preg_replace(array('/[a-zA-Z]/','/ /','/,/'),'',$value);
		$this->$key = intval($value);
	}
	
	function setNum($key, $value)
	{
		$value = preg_replace(array('/[a-zA-Z]/','/ /','/,/'),'',$value);
		$this->$key = abs(intval($value));
	}
	
	function setDec($key, $value)
	{
		$value = preg_replace(array('/[a-zA-Z]/','/ /','/,/'),'',$value);
		// echo '{'.$value.'}'.floatval($value);
		$this->$key = floatval($value);
	}
	
	function setStr($key, $value)
	{
		$this->$key = strip_tags($value);
	}
	
	function setTxt($key, $value)
	{
		$this->$key = 
			preg_replace("/<script[^>]*>[^<]+<\/script[^>]*>/is","", $value);
	}
	
	function setEml($key, $value)
	{
		if ($value) {
			if (preg_match("/^[a-z0-9]([a-z0-9_\-\.]*)@([a-z0-9_\-\.]*)\.([a-z]{2,4})$/i",$value))
			{
				$this->$key = $value;
			} else {
				$this->e($key,"invalid email");
			}
		} else if (!is_null($value)) {
			$this->$key = '';
		}
	}
	
	function setUrl($key, $value)
	{
		if ($value && (!eregi("^(http|https|ftp)?://",$value))) {
			$value = "http://".$value;
		}
		if ($value == "http://") {
			$value = "";
		}
		$this->$key = $value;
	}
	
	function setBbs($key, $value)
	{
		$value = preg_replace("/<script[^>]*>[^<]+<\/script[^>]*>/is"
			,"", $value); 
		$value = preg_replace("/<\/?(div|span|iframe|frame|input|"
			."textarea|script|style|applet|object|embed|form)[^>]*>/is"
			,"", $value);
		$this->$key = $value;
	}
	
	function setHtm($key, $value)
	{
		// scary
		$this->$key = $value;
	}
	
	function setBol($key, $value)
	{
		$this->$key = $value?1:0;
	}
	
	function setDte($key, $value)
	{
		$this->$key = Tzn::_dteValue($value);
	}
	
	function setDtm($key, $value, $tz=TZN_TZSERVER)
	{
		$this->$key = Tzn::_dtmValue($value,'SQL',$tz);
	}
	
	function setObj($key, $value)
	{
		$class = ucFirst($key);
		$obj = new $class;
		$obj->setAuto($value,$key); // nested
		$this->$key = $obj;
	}
    
	/* --- PRINT  ------------------------------------------------ */
	// this is where i'm losing it (nobody needs that crap)
	
	function p($key)
	{
		/* -TODO- redirect to corresponding method */
		$arrArgs = func_get_args();
		print call_user_func_array(array(&$this,'get'), $arrArgs);
	}
		
	function pStr($keyval)
	{
		$arg =  func_get_args();
		$pos = 1;
		if (is_int($arg[1])) {
			$cut = $arg[1];
			$pos++;
		}
		$default = $arg[$pos++];
		$style = $arg[$pos];
		print Tzn::_strValue($keyval,$cut,$default);
	}

	function pNum($keyval,$default='')
	{
		print Tzn::getNum($keyval,$default);
	}
	
	function pEml($keyval,$default='',$style='')
	{
		$value = Tzn::_value($keyval);
		if ($value) {
			print '<a href="mailto:'.$value.'"'.Tzn::_style($style)
				.'">'.$value.'</a>';
		} else {
			print $default;
		}
	}
	
	function pUrl($keyval,$target='',$default='',$style='',$xtra='')
	{
		$value = Tzn::_value($keyval);
		if ($value) {
			$str = '<a href="'.$value.'" '.Tzn::_style($style);
			if ($target) {
				$str .= 'target="'.$target.'" ';
			}
			if ($xtra) {
				$str .= $xtra;
			}
			$str .= '>'.preg_replace("/^(http[s]?|ftp):\/\//i",''
				,$value).'</a>';
			print $str;
		} else {
			print $default;
		}
	}
	
	function pTxt($keyval)
	{
		$arg =  func_get_args();
		$pos = 1;
		if (is_int($arg[1])) {
			$cut = $arg[1];
			$pos++;
		}
		$default = $arg[$pos++];
		$style = $arg[$pos];
		print nl2br(Tzn::_strValue($keyval,$cut,$default));
	}
	
	function pBbs($keyval,$default='')
	{
		$value = Tzn::_value($keyval,$default);
		$value = preg_replace("/(?<!<a href=\")((http|ftp)+(s)?"
			.":\/\/[^<>\s]+)/i", "<a href=\"\\0\">\\0</a>", $value);
		print nl2br($value);
	}
	
	function pHtm($keyval,$default='&nbsp;')
	{
		print Tzn::_value($keyval,$default);
	}
	
	function pBol($keyval,$default='',$yes=TZN_BOOL_TRUE,$no=TZN_BOOL_FALSE)
	{
		$value = Tzn::getBol($keyval,$default);
		print $value?$yes:$no;
	}
		
	function pDte($keyval,$format='SQL',$default='')
	{
		print Tzn::getDte($keyval,$format,$default);
	}
	
	function pDtm($keyval,$format='SQL',$tz=TZN_TZDEFAULT,$default='-')
	{
		if (ereg('^(0000|9999)',$value)) {
			$value = $default;
		} else {
			$value = Tzn::getDtm($keyval,$format,$tz,$default);
		}
		print $value;
	}
	
	function pTmz($keyval,$default='') 
	{
		$value = $keyval;
		if (is_object($this)) {
			$value = $this->_value($keyval);
		}
		print Tzn::_tmz($value);
	}
	
	function _tmz($tz)
	{
		$str = "GMT";
        if ($tz != null) {
            $tza = abs($tz);
            $tzHour = ($tza / 3600);
            $tzMin = ($tza % 3600);
            if ($tz >= 0) {
                $str .= "+";
            } else {
                $str .= "-";
            }
            if ($tzHour < 10) {
                $str .= "0";
            }
            $str .= $tzHour;
            if ($tzMin < 10) {
                $str .= "0";
            }
            $str .= $tzMin;
        }
        return $str;
	}
	
	/**
	 * _style : create class/style element
	 * @access private 
	 */
	function _style($style)
	{
		if ($style) {
			if (strpos($style,':')) {
				// style definition eg. color:#fff
				$slabel = 'style';
			} else {
				$slabel = 'class';
			}
			return $slabel.'="'.$style.'" ';
		} else {
			return '';
		}
	}
	
	/* --- Error Methods -------------------------------------------- */
	
	function checkEmpty($fields) {
		$arrFields = explode(',',$fields);
		foreach ($arrFields as $field) {
			$field = trim($field);
			if ((!$this->$field) 
				|| ((is_object($this->$field)) 
				&& (!$this->$field->id)))
			{
				$this->e($field,'this information is compulsory');
			}
		}
		return (count($this->_error) == 0);
	}

	/**
	 * e get error
	 */
    function e($key,$value='')
    {
        if ($key) {
        	if ($value) {
        		$this->_error[$key] = $value;
        	} else {
	            return $this->_error[$key];
	        }
        }
    }
    /**
	 * gError 
	 * an alias for e()
	 */
    function gError($key)
    {
    	$this->e($key,$value);
    }
    
    /**
	 * print error
	 * this is the function you may redefine in sub classes
	 */
    function printError($key)
    {
        if ($this->_error[$key]) {
            print '<span class="tznError">'.$this->_error[$key].'</span>';
        }
    }
    
    /**
     * pError
     * an alias for printError, for both static use and object instance
     */
    function pError($key)
    {
    	if (is_object($this)) {
	        $this->printError($key);
	    }
    }

    function printErrorList()
    {
        // usually used for debug
		if (count($this->_error)) {
			forEach($this->_error as $key => $value) {
				echo $key." =&gt; ".$value."<br/>";
			}
		}
    }
        
    /* --- Miscellaneous  --------------------------------- */

	function qSelect($name,$keyval,$default='',$nochoice='',$xtra='')
	{
		$form = '<select name="'.$name.'" ';
		if ($xtra) {
			$form .= $xtra.' ';
		}
		$form .= '>';
		if ($nochoice) {
			$form .='<option value="">'.$nochoice.'</option>';
		}
		if (is_object($this)) {
			if ($this->rMore()) {
				while ($item = $this->rNext()) { 
					$form .= '<option';
					$value = $item->_value($keyval);
					if ($value == $default) {
						$form .= ' selected="true"';
					}
					$form .= '>'.$value.'</option>';
				}
			}
		}
		$form .= '</select>';
		print $form;
		Tzn::pError($name);
	}
    
    function qSession()
    {
    	if (@constant('TZN_TRANS_ID')) {
			print '<input type="hidden" name="'.ini_get('session.name')
    			.'" value="'.session_id().'" />';
    	}
    }

	function getHttpParameter($method,$key,$save=false,$recover=false) {
		$arrValue = array();
		switch($method) {
			case 'get':
				$arrValue =& $_GET;
				break;
			case 'post':
				$arrValue =& $_POST;
				break;
			default:
				$arrValue =& $_REQUEST;
				break;
		}
		$value = null;
		if ($value = $arrValue[$key]) {
			// got from HTTP query string
			if (get_magic_quotes_gpc() && $value) {
				// remove fucking magic quotes
				$value=stripslashes($value);
			}
		} else if ($recover) {
			$value = $_SESSION[$key];
		}
		if ($save) {
			// save to session
			if ($value) {
				$_SESSION[$key] = $value;
			} else {
				unset($_SESSION[$key]);
			}
		}
		return $value;
	}
    
    function redirect($url,$message='')
    {
    	if (@constant('TZN_TRANS_ID')) {
			if (!ereg(session_id(),$url)) {
				$url = Tzn::concatUrl($url,session_name()
					.'='.session_id());
			}
    	}
    	if ($message) {
    		$url = Tzn::concatUrl($url,'tznMessage='.urlencode($message));
    	}
    	header("Location: ".$url);
    	exit;
    }
    
    function concatUrl($url,$param)
    {
		if ($pos = strpos($url,'?')) {
			$arrParam = explode('=',$param);
			if (strpos($url,$arrParam[0].'=')) {
				// parameter already in url
				$strQuery = substr($url,$pos+1);
				$arrQuery = explode('&',$strQuery);
				$arrResult = array();
				$found = false;
				foreach ($arrQuery as $value) {
					if (ereg('^'.$arrParam[0].'=', $value)) {
						$arrResult[] = $param;
						$found = true;
					} else {
						$arrResult[] = $value;
					}
				}
				if ($found) {
					$url = substr($url,0,$pos).'?'.implode('&',$arrResult);
				} else {
					$url .= '&'.$param;
				}
			} else {
				$url .= '&'.$param;
			}
    	} else {
    		$url .= '?'.$param;
    	}
    	return $url;
    }
	
	
}
